@@ -24,7 +24,7 @@ fn main() {
         Err(e) => e,
     };
 
-    eprintln!("Error launching `probe-rs`: {}", err);
+    eprintln!("Error launching `probe-rs`: {err}");
     eprintln!("Note: the `cargo-embed` binary is a small shim that launches `probe-rs`.");
     eprintln!("Make sure `probe-rs` is installed and available in $PATH.");
 
