@@ -1,5 +1,4 @@
 mod config;
-mod error;
 mod rttui;
 
 use crate::cmd::gdb_server::GdbInstanceConfiguration;
@@ -45,7 +44,7 @@ use crate::util::{cargo::build_artifact, common_options::CargoOptions, logging};
 struct CliOptions {
     /// Name of the configuration profile to use.
     #[arg()]
-    config: Option<String>,
+    config_profile: Option<String>,
     /// Path of a configuration file outside the default path.
     ///
     /// When this is set, the default path is still considered, but the given file is considered
@@ -123,7 +122,7 @@ async fn main_try(args: &[OsString], offset: UtcOffset) -> Result<()> {
     let work_dir = std::env::current_dir()?;
 
     // Get the config.
-    let config_name = opt.config.as_deref().unwrap_or("default");
+    let profile_name = opt.config_profile.as_deref().unwrap_or("default");
     let mut configs = config::Configs::new(work_dir.clone());
     if let Some(config_file) = opt.config_file {
         let config_file = PathBuf::from(config_file);
@@ -134,7 +133,7 @@ async fn main_try(args: &[OsString], offset: UtcOffset) -> Result<()> {
         }
         configs.merge(config_file)?;
     }
-    let config = configs.select_defined(config_name)?;
+    let config = configs.select_defined(profile_name)?;
 
     let _log_guard = setup_logging(None, config.general.log_level);
 
@@ -167,9 +166,13 @@ async fn main_try(args: &[OsString], offset: UtcOffset) -> Result<()> {
         )
     })?;
 
-    logging::println(format!("      {} {}", "Config".green().bold(), config_name));
     logging::println(format!(
         "      {} {}",
+        "Profile".green().bold(),
+        profile_name
+    ));
+    logging::println(format!(
+        "       {} {}",
         "Target".green().bold(),
         path.display()
     ));
@@ -287,6 +290,7 @@ async fn main_try(args: &[OsString], offset: UtcOffset) -> Result<()> {
             flash_layout_output_path: None,
             preverify: config.flashing.preverify,
             verify: config.flashing.verify,
+            chip_erase: config.flashing.do_chip_erase,
         };
         let format_options = FormatOptions::default();
         let loader = build_loader(&mut session, &path, format_options, image_instr_set)?;
@@ -301,7 +305,6 @@ async fn main_try(args: &[OsString], offset: UtcOffset) -> Result<()> {
             &download_options,
             &probe_options,
             loader,
-            config.flashing.do_chip_erase,
         )?;
 
         match boot_info {
@@ -342,7 +345,7 @@ async fn main_try(args: &[OsString], offset: UtcOffset) -> Result<()> {
                 gdb_connection_string.as_deref().unwrap_or("127.0.0.1:1337");
 
             logging::println(format!(
-                "    {} listening at {}",
+                "     {} listening at {}",
                 "GDB stub".green().bold(),
                 gdb_connection_string,
             ));
@@ -377,16 +380,16 @@ async fn main_try(args: &[OsString], offset: UtcOffset) -> Result<()> {
     }
 
     logging::println(format!(
-        "        {} processing config {}",
+        "        {} processing config profile {}",
         "Done".green().bold(),
-        config_name
+        profile_name,
     ));
 
     Ok(())
 }
 
 fn should_resume_core(config: &config::Config) -> bool {
-    if config.flashing.enabled {
+    if config.flashing.enabled && !config.reset.halt_afterwards {
         true
     } else {
         !(config.reset.enabled && config.reset.halt_afterwards)
