@@ -149,10 +149,9 @@ pub(crate) fn disassemble_target_memory(
             .core_data
             .debug_info
             .get_source_location(start_from_address)
+            && let Some(source_address) = source_location.address
         {
-            if let Some(source_address) = source_location.address {
-                start_from_address = source_address;
-            }
+            start_from_address = source_address;
         }
     }
 
@@ -242,6 +241,7 @@ pub(crate) fn disassemble_target_memory(
                 line: None,
                 location: None,
                 symbol: None,
+                presentation_hint: None,
             });
             instruction_pointer += min_instruction_size;
             continue 'instruction_loop;
@@ -272,6 +272,7 @@ pub(crate) fn disassemble_target_memory(
                         line: None,
                         location: None,
                         symbol: None,
+                        presentation_hint: None,
                     });
                     code_buffer_le = code_buffer_le
                         .split_at(min_instruction_size as usize)
@@ -332,12 +333,13 @@ pub(crate) fn disassemble_target_memory(
                         instruction
                             .bytes()
                             .iter()
-                            .map(|b| format!("{:02X}", b))
+                            .map(|b| format!("{b:02X}"))
                             .join(" "),
                     ),
                     line,
                     location,
                     symbol: None,
+                    presentation_hint: None,
                 });
 
                 code_buffer_le = code_buffer_le.split_at(instruction.len()).1.to_vec();
@@ -360,14 +362,13 @@ pub(crate) fn disassemble_target_memory(
         let maybe_inst_with_location = disassembled_instructions
             .drain(0..first_instruction_index)
             .rfind(|inst| inst.location.is_some());
-        if let Some(inst_with_location) = maybe_inst_with_location {
-            if let Some(first_instruction) = disassembled_instructions.get_mut(0) {
-                if first_instruction.location.is_none() {
-                    first_instruction.line = inst_with_location.line;
-                    first_instruction.column = inst_with_location.column;
-                    first_instruction.location = inst_with_location.location;
-                }
-            }
+        if let Some(inst_with_location) = maybe_inst_with_location
+            && let Some(first_instruction) = disassembled_instructions.get_mut(0)
+            && first_instruction.location.is_none()
+        {
+            first_instruction.line = inst_with_location.line;
+            first_instruction.column = inst_with_location.column;
+            first_instruction.location = inst_with_location.location;
         }
     } else {
         return Err(DebuggerError::Other(anyhow!(
@@ -418,7 +419,7 @@ fn get_capstone_le(target_core: &mut CoreHandle) -> Result<Capstone, DebuggerErr
             .build(),
         InstructionSet::Xtensa => return Err(DebuggerError::Unimplemented),
     }
-    .map_err(|err| anyhow!("Error creating capstone: {:?}", err))?;
+    .map_err(|err| anyhow!("Error creating capstone: {err:?}"))?;
     let _ = cs.set_skipdata(true);
     Ok(cs)
 }
@@ -436,10 +437,10 @@ pub(crate) fn get_dap_source(source_location: &SourceLocation) -> Option<Source>
     let native_path = file_path.with_windows_encoding();
     let native_path = std::path::PathBuf::try_from(native_path)
         .map(|mut path| {
-            if path.is_relative() {
-                if let Ok(current_dir) = std::env::current_dir() {
-                    path = current_dir.join(path);
-                }
+            if path.is_relative()
+                && let Ok(current_dir) = std::env::current_dir()
+            {
+                path = current_dir.join(path);
             }
             path
         })
@@ -461,23 +462,21 @@ pub(crate) fn get_dap_source(source_location: &SourceLocation) -> Option<Source>
 
     // Precompiled rustlib paths start with /rustc/<hash>/ which needs to be
     // mapped to <sysroot>/lib/rustlib/src/rust/
-    if let Some((old_prefix, new_prefix)) = RUSTLIB_SOURCE_MAP.as_ref() {
-        if let Ok(path) = file_path.strip_prefix(old_prefix) {
-            if let Ok(rustlib_path) = std::path::PathBuf::try_from(new_prefix.join(path)) {
-                if rustlib_path.exists() {
-                    return Some(Source {
-                        name: file_name,
-                        path: Some(rustlib_path.to_string_lossy().to_string()),
-                        source_reference: None,
-                        presentation_hint: None,
-                        origin: None,
-                        sources: None,
-                        adapter_data: None,
-                        checksums: None,
-                    });
-                }
-            }
-        }
+    if let Some((old_prefix, new_prefix)) = RUSTLIB_SOURCE_MAP.as_ref()
+        && let Ok(path) = file_path.strip_prefix(old_prefix)
+        && let Ok(rustlib_path) = std::path::PathBuf::try_from(new_prefix.join(path))
+        && rustlib_path.exists()
+    {
+        return Some(Source {
+            name: file_name,
+            path: Some(rustlib_path.to_string_lossy().to_string()),
+            source_reference: None,
+            presentation_hint: None,
+            origin: None,
+            sources: None,
+            adapter_data: None,
+            checksums: None,
+        });
     }
 
     // If no matching file was found
@@ -580,6 +579,7 @@ pub(crate) fn set_instruction_breakpoint(
         offset: None,
         source: None,
         verified: false,
+        reason: None,
     };
 
     if let Ok(MemoryAddress(memory_reference)) = requested_breakpoint
