@@ -7,13 +7,11 @@ pub(crate) use adi_memory_interface::ADIMemoryInterface;
 
 use crate::{CoreStatus, memory::MemoryInterface, probe::DebugProbeError};
 
-use super::{
-    ArmError, ArmProbeInterface, DapAccess, FullyQualifiedApAddress,
-    communication_interface::SwdSequence,
-};
+use super::{ArmDebugInterface, ArmError, FullyQualifiedApAddress};
 pub use romtable::{Component, ComponentId, CoresightComponent, PeripheralType, RomTable};
 
-/// An ArmMemoryInterface (ArmProbeInterface + MemoryAp)
+/// Trait for accessing memory behind a memory access port,
+/// as defined in the ARM Debug Interface Specification.
 pub trait ArmMemoryInterface: MemoryInterface<ArmError> {
     /// The underlying MemoryAp address.
     fn fully_qualified_address(&self) -> FullyQualifiedApAddress;
@@ -21,14 +19,8 @@ pub trait ArmMemoryInterface: MemoryInterface<ArmError> {
     /// The underlying memory AP’s base address.
     fn base_address(&mut self) -> Result<u64, ArmError>;
 
-    /// Get this interface as a SwdSequence object.
-    fn get_swd_sequence(&mut self) -> Result<&mut dyn SwdSequence, DebugProbeError>;
-
-    /// Get this interface as a [`ArmProbeInterface`] object.
-    fn get_arm_probe_interface(&mut self) -> Result<&mut dyn ArmProbeInterface, DebugProbeError>;
-
-    /// Get this interface as a [`DapAccess`] object.
-    fn get_dap_access(&mut self) -> Result<&mut dyn DapAccess, DebugProbeError>;
+    /// Get this interface as a [`ArmDebugInterface`] object.
+    fn get_arm_debug_interface(&mut self) -> Result<&mut dyn ArmDebugInterface, DebugProbeError>;
 
     /// Get the current value of the CSW reflected in this probe.
     fn generic_status(&mut self) -> Result<crate::architecture::arm::ap::CSW, ArmError>;
